/* ---------------------------------------------------------- */
/* (C)2000 Blue Chip Technology Ltd.  All Rights Reserved.    */
/* ---------------------------------------------------------- */
/*                                                            */
/*  FileName  : $WorkFile::                                 $ */
/*                                                            */
/*  Release   : 1.00                                          */
/*                                                            */
/*  Contents  : This file contains the functions for the C    */
/*              based console mode application that will      */
/*              test the PCI-ADC input and output functions   */
/*              using the pacer and by direct calls.          */
/*                                                            */
/*  History   :                                               */
/*  $Revision::                                             $ */
/*  $Author::                                               $ */
/*  $Modtime::                                              $ */
/*  $History::                                              $ */
/*                                                            */
/* ---------------------------------------------------------- */

/* ---------------------------------------------------------- */
/* Standard Windows console mode include files.               */
/* ---------------------------------------------------------- */

#include <windows.h>
#include <winioctl.h>
#include <time.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>


/* ---------------------------------------------------------- */
/* Application specific include files.                        */
/* ---------------------------------------------------------- */

#include "bluechip.h"


/* ---------------------------------------------------------- */
/* Program name and version number strings.                   */
/* ---------------------------------------------------------- */

#define PROGNAME  "PCI Data Acquisition PCI-ADC Test Program v"
#define VERSION   "1.00.00"
#define COPYRIGHT "(C)2000 Blue Chip Technology Ltd."
#define RIGHTS    "All rights reserved"


/* ---------------------------------------------------------- */
/* Handles and board ID structures.                           */
/* ---------------------------------------------------------- */

BCT_HANDLE nPciAdcPortAHandle;
BCT_HANDLE nPciAdcPortBHandle;
BCT_HANDLE nPciAdcPortCHandle;
BCT_HANDLE nPciAdcAnalInHandle;
BCT_HANDLE nPciAdcAnalOutHandle;
BCT_BOARD_ID pPciAdcBoardId;
PBCT_BUFFER pAdcBuffer;


/* ---------------------------------------------------------- */
/* Define keystrokes.                                         */
/* ---------------------------------------------------------- */

#define	ESC 0x1B


/* ---------------------------------------------------------- */
/* Local function prototypes.                                 */
/* ---------------------------------------------------------- */

void Initialise(void);
void Deinitialise(void);
void DirectIO(void);
void PacerIO(void);
void AutoCal(void);
void AnalIn(void);
void MultiAnalIn(void);
void AnalOut(void);


/* ---------------------------------------------------------- */
/* Main function.                                             */
/*                                                            */
/* Input  : None                                              */
/* Return : Error code if problems.                           */
/* ---------------------------------------------------------- */

int main(void)
{
	char cSelection;

	// Display the program header
	printf("%s%s\n%s\n%s\n", PROGNAME, VERSION, COPYRIGHT, RIGHTS);

	// Initialise the handles and the board ID structure.
	Initialise();

	// Display a menu and wait for a keypress
	do
	{
		printf("\n\n");
		printf("Please select a test:\n");
		printf("\t   1)  Perform direct IO loopback test PIO port C -> port C\n");
		printf("\t   2)  Perform pacer controlled IO loopback test PIO port A -> port B\n");
		printf("\t   3)  Perform Auto calibration of Analogue input\n");
		printf("\t   4)  Perform Analogue input from channel 0 using pacer control\n");
		printf("\t   5)  Perform multiple analogue inputs from channel 0 using pacer control\n");
		printf("\t   6)  Perform Analogue output on channel 0\n");
		printf("\t<ESC> Exit\n");
		cSelection = _getch();
		switch (cSelection)
		{
			case '1':
				DirectIO();
				break;
			case '2':
				PacerIO();
				break;
			case '3':
				AutoCal();
				break;
			case '4':
				AnalIn();
				break;
			case '5':
				MultiAnalIn();
				break;
			case '6':
				AnalOut();
				break;
		}
	} while (cSelection != ESC);

	// We are done so close the handles and release the board ID
	Deinitialise();
}


/* ---------------------------------------------------------- */
/* Local function implementations.                            */
/* ---------------------------------------------------------- */

/* ---------------------------------------------------------- */
/* Initialise - Initialise the board id structure and the     */
/*              handles that we are going to use.             */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void Initialise(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	int nIndex;

	// Initialise the five handles we are going to use
	dwStatus = BCTInitHandle(&nPciAdcPortAHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitHandle returned: %s\n", szErrorText);
	}
	dwStatus = BCTInitHandle(&nPciAdcPortBHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitHandle returned: %s\n", szErrorText);
	}
	dwStatus = BCTInitHandle(&nPciAdcPortCHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitHandle returned: %s\n", szErrorText);
	}
	dwStatus = BCTInitHandle(&nPciAdcAnalInHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitHandle returned: %s\n", szErrorText);
	}
	dwStatus = BCTInitHandle(&nPciAdcAnalOutHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitHandle returned: %s\n", szErrorText);
	}

	// Get the driver to fill out the board ID structure
	dwStatus = BCTGetBoardId(&pPciAdcBoardId, PCI_ADC, 0);
	if (dwStatus != BCT_OK)
	{

		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTGetBoardId returned: %s\n", szErrorText);
	}

	// The loopback connector that we use with this test is 
	// configured that PIO 0 port A is looped back to PIO 0
	// port B and PIO 0 port C upper nibble is looped back to 
	// PIO 0 port C lower nibble.
	// As such set the port directions up to do output on the PIO
	// so that port A is output, port B is input and port C is set
	// for output on the upper nibble and input on the lower
	// nibble.
	dwStatus = BCTInit8255Modes(&pPciAdcBoardId, 0, MODE_0, OUTPUT, INPUT, OUTIN);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInit8255Modes returned: %s\n", szErrorText);
	}

	// Open the handles that will be used by the IO calls we will make
	dwStatus = BCTOpen(&nPciAdcPortAHandle, &pPciAdcBoardId, BCT_8255, 0, BCT_PORT_A);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTOpen returned: %s\n", szErrorText);
	}
	dwStatus = BCTOpen(&nPciAdcPortBHandle, &pPciAdcBoardId, BCT_8255, 0, BCT_PORT_B);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTOpen returned: %s\n", szErrorText);
	}
	dwStatus = BCTOpen(&nPciAdcPortCHandle, &pPciAdcBoardId, BCT_8255, 0, BCT_PORT_C);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTOpen returned: %s\n", szErrorText);
	}
	dwStatus = BCTOpen(&nPciAdcAnalInHandle, &pPciAdcBoardId, BCT_AIN, 0, 0);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTOpen returned: %s\n", szErrorText);
	}

	// Before we can open the handles for the analogue output we need to
	// set whether they are voltage or current sources.
	dwStatus = BCTInitAOutModes(&pPciAdcBoardId, 0, BCT_AOUT_VOLTAGE, BCT_AOUT_VOLTAGE, BCT_AOUT_VOLTAGE, BCT_AOUT_VOLTAGE);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitAOutModes returned: %s\n", szErrorText);
	}
	dwStatus = BCTOpen(&nPciAdcAnalOutHandle, &pPciAdcBoardId, BCT_AOUT, 0, 0);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTOpen returned: %s\n", szErrorText);
	}

	// Allocate some space and fill in the buffer structure so that we 
	// have data for the paced output
	dwStatus = BCTAllocate(&pAdcBuffer, 256);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAllocate returned: %s\n", szErrorText);
	}
	for (nIndex = 0; nIndex < 256; nIndex++)
	{
		pAdcBuffer -> Buffer[nIndex] = nIndex;
	}
}


/* ---------------------------------------------------------- */
/* Deinitialise - Close the board id structure and the        */
/*                handles that we have been using.            */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void Deinitialise(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];

	// Make sure that the memory we have been using for the pacer
	// is released
	dwStatus = BCTRelease(&pAdcBuffer);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTRelease returned: %s\n", szErrorText);
	}

	// Close the handles
	dwStatus = BCTClose(&nPciAdcPortAHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTClose returned: %s\n", szErrorText);
	}
	dwStatus = BCTClose(&nPciAdcPortBHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTClose returned: %s\n", szErrorText);
	}
	dwStatus = BCTClose(&nPciAdcPortCHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTClose returned: %s\n", szErrorText);
	}
	dwStatus = BCTClose(&nPciAdcAnalInHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTClose returned: %s\n", szErrorText);
	}
	dwStatus = BCTClose(&nPciAdcAnalOutHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTClose returned: %s\n", szErrorText);
	}

	// Release the board ID structure
	dwStatus = BCTReleaseBoardId(&pPciAdcBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTReleaseBoardId returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* DirectIO - Perform the loopback test using PIO 0 port C    */
/*            upper nibble to PIO 0 port C lower nibble       */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void DirectIO(void)
{
	BCT_BYTE nOutVal;
	BCT_BYTE nInVal;
	BCT_DWORD dwStatus;
	long int loutval;

	char szErrorText[120];

	// Get a value from the user to output to the PIO
	do
	{
		printf("Please enter a value in hex to output to PIO 0 port C upper nibble: ");
		scanf_s("%x", &loutval);
		nOutVal = (BCT_BYTE) loutval;
	} while (nOutVal < 0x0 && nOutVal > 0xF);

	// Output the value to PIO 0 port C upper nibble
	nOutVal = nOutVal << 4;
	dwStatus = BCTWritePort(&nPciAdcPortCHandle, nOutVal);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciAdcPortCHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTWritePort returned: %s\n", szErrorText);
	}

	// Read it back to ensure that we got what we were expecting
	dwStatus = BCTReadPort(&nPciAdcPortCHandle, &nInVal);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciAdcPortCHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTReadPort returned: %s\n", szErrorText);
	}

	// Comapre the values
	nInVal &= 0x0F;
	printf("Output 0x%.1x, read back 0x%.1x. %s\n", nOutVal >> 4, nInVal, (nOutVal >> 4) == nInVal ? "CORRECT" : "INCORRECT");
}


/* ---------------------------------------------------------- */
/* PacerIO - Perform the loopback test using PIO 0 port A and */
/*           B using the pacer to drive data from the buffer  */
/*           to the output.                                   */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void PacerIO(void)
{
	BCT_BYTE nInVal;
	BCT_DWORD dwStatus;
	char szErrorText[120];
	char cDummy;

	// Tell the user what is going on.
	printf("Starting pacer...press any key to exit\n");

	// Initialise a pacer
	dwStatus = BCTInitPacer(&pPciAdcBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitPacer returned: %s\n", szErrorText);
	}

	// Add an event to the pacers event list
	dwStatus = BCTAddPacerBlockIo(&nPciAdcPortAHandle, &pPciAdcBoardId, BLOCK_REPEATED_WRITE, pAdcBuffer, NULL, 256, 0);
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAddPacerBlockIo returned: %s\n", szErrorText);
	}

	// Start the pacer itself
	dwStatus = BCTStartPacer(&pPciAdcBoardId, 1000);    // Update at 1s interval
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTStartPacer returned: %s\n", szErrorText);
	}

	// Spin in a loop until the user presses a key
	while (!_kbhit())
	{
		dwStatus = BCTReadPort(&nPciAdcPortBHandle, &nInVal);
		if (dwStatus == BCT_IO_PENDING)
		{
			dwStatus = BCTWait(&nPciAdcPortBHandle, INFINITE);
		}
		if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
		{
			BCTErr2Txt(dwStatus, szErrorText);
			printf("BCTReadPort returned: %s\n", szErrorText);
		}

		// Display the value we have read back
		printf("Value read from PIO 0 port B: 0x%.2x\r", nInVal);
	}
	cDummy = _getch();             // Throw away the dummy keypress
	printf("\n");

	// Stop the pacer before we exit
	dwStatus = BCTStopPacer(&pPciAdcBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTStopPacer returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* AutoCal - Get the results of the autocalibration function  */
/*           of the PCI-ADC analogue inputs.                  */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void AutoCal(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	BCT_INT16 wMeanZero;
	BCT_INT16 wMeanFsd;

	// Call the auto calibrate function with a gain of 1.  We have
	// already opened the handle.
	dwStatus = BCTAutoCalAin(&nPciAdcAnalInHandle, PCIADC_AIN_GAIN_1, &wMeanZero, &wMeanFsd);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciAdcAnalInHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAutoCalAin returned: %s\n", szErrorText);
	}
	if (dwStatus == BCT_OK)
	{
		printf("Mean zero 0x%.4x, mean FSD 0x%.4x\n", wMeanZero, wMeanFsd);
	}

	// We will close the handle when we shut the application down so no need
	// to do it here.
}


/* ---------------------------------------------------------- */
/* AnalIn - Read an analogue voltage from the input pins of   */
/*          analogue input channel 0.                         */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void AnalIn(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	PBCT_BUFFER pAnalogueBuffer;
	BCT_WORD *pBuf;
	int nIndex;

	// We have already opened the handle that we are going to 
	// be using for capturing the analogue input data.

	// Allocate the space required by the buffer
	dwStatus = BCTAllocate(&pAnalogueBuffer, 2048);   // Make sure we have plenty of buffer
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAllocate returned: %s\n", szErrorText);
	}

	// Get a pacer for sampling the data
	dwStatus = BCTAcquireAPacer(&pPciAdcBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAcquireAPacer returned: %s\n", szErrorText);
	}

	// Capture the data for input 0, gain = 1, in single ended mode
	dwStatus = BCTReadBlockAin(&pPciAdcBoardId, &nPciAdcAnalInHandle, 0, PCIADC_AIN_GAIN_1, PCIADC_AIN_MODE_SINGLE, pAnalogueBuffer, 500, 10000);
	if (dwStatus == BCT_IO_PENDING)
	{
		BCTWait(&nPciAdcAnalInHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTReadBlockAin returned: %s\n", szErrorText);
	}

	// Display the first few entries from the buffer
	pBuf = (BCT_WORD *)pAnalogueBuffer -> Buffer;
	for (nIndex = 0; nIndex < 10; nIndex++)
	{
		printf("Sample[%d] = 0x%.4x\n", nIndex, pBuf[nIndex]);
	}

	// Release the buffer
	dwStatus = BCTRelease(&pAnalogueBuffer);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAllocate returned: %s\n", szErrorText);
	}

	// Release the pacer as we are done with it.
	dwStatus = BCTReleaseAPacer(&pPciAdcBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTReleaseAPacer returned: %s\n", szErrorText);
	}

	// We will close the handles when we terminate the application
}


void MultiAnalIn(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	PBCT_BUFFER pAnalogueBuffer;
	BCT_WORD *pBuf;
	int nIndex;
	int nCount;

	// We have already opened the handle that we are going to 
	// be using for capturing the analogue input data.

	printf("Acquiring data...\n");

	// Allocate the space required by the buffer
	dwStatus = BCTAllocate(&pAnalogueBuffer, 2048);   // Make sure we have plenty of buffer
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAllocate returned: %s\n", szErrorText);
	}

	// Get a pacer for sampling the data
	dwStatus = BCTAcquireAPacer(&pPciAdcBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAcquireAPacer returned: %s\n", szErrorText);
	}

	for (nCount = 0; nCount < 10; nCount++)
	{
		// Capture the data for input 0, gain = 1, in single ended mode
		dwStatus = BCTReadBlockAin(&pPciAdcBoardId, &nPciAdcAnalInHandle, 0, PCIADC_AIN_GAIN_1, PCIADC_AIN_MODE_SINGLE, pAnalogueBuffer, 500, 10000);
		if (dwStatus == BCT_IO_PENDING)
		{
			BCTWait(&nPciAdcAnalInHandle, INFINITE);
		}
		if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
		{
			BCTErr2Txt(dwStatus, szErrorText);
			printf("BCTReadBlockAin returned: %s\n", szErrorText);
		}

		// Display the first few entries from the buffer
		pBuf = (BCT_WORD *)pAnalogueBuffer -> Buffer;
		for (nIndex = 0; nIndex < 10; nIndex++)
		{
			printf("0x%.4x ", pBuf[nIndex]);
			pBuf[nIndex] = 0xFFFF;            // Set some default value into the buffer
		}
		printf("\n");
	}

	// Release the buffer
	dwStatus = BCTRelease(&pAnalogueBuffer);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAllocate returned: %s\n", szErrorText);
	}

	// Release the pacer as we are done with it.
	dwStatus = BCTReleaseAPacer(&pPciAdcBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTReleaseAPacer returned: %s\n", szErrorText);
	}

	// We will close the handles when we terminate the application
}


/* ---------------------------------------------------------- */
/* AnalOut - Drive a voltage onto the output pins of          */
/*           analogue output channel 0.                       */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void AnalOut(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	BCT_WORD wVoltage;
	long int loutval;

	// The handle we are going to use for analogue output has
	// already been opened and the outputs have been set as 
	// voltage sources.

	// Get a value from the user for the voltage to be output
	// and scale it to the appropriate value between 0x000 and
	// 0xFFF.
	do
	{
		printf("Please enter a voltage between -10V and 10V to output to channel 0: ");
		scanf_s("%x", &loutval);
		wVoltage = (BCT_WORD) loutval;
	} while (wVoltage < -10 && wVoltage > 10);
	wVoltage += 10;               // Convert -10V to + 10V to 0 to 20
	wVoltage = (BCT_WORD)(4095 * wVoltage / 20);

	// Write the value to the analogue output port
	dwStatus = BCTWritePort16(&nPciAdcAnalOutHandle, wVoltage);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciAdcAnalOutHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTWritePort16 returned: %s\n", szErrorText);
	}

	// We will close the handle when we shut the application down so no need
	// to do it here.
}


/* ---------------------------------------------------------- */
/* End of listing.                                            */
/* ---------------------------------------------------------- */
