/* ---------------------------------------------------------- */
/* (C)2000 Blue Chip Technology Ltd.  All Rights Reserved.    */
/* ---------------------------------------------------------- */
/*                                                            */
/*  FileName  : $WorkFile::                                 $ */
/*                                                            */
/*  Release   : 1.00                                          */
/*                                                            */
/*  Contents  : This file contains the functions for the C    */
/*              based console mode application that will      */
/*              perform direct controller or pacer driven     */
/*              IO on a PCI-DIO.                              */
/*                                                            */
/*  History   :                                               */
/*  $Revision::                                             $ */
/*  $Author::                                               $ */
/*  $Modtime::                                              $ */
/*  $History::                                              $ */
/*                                                            */
/* ---------------------------------------------------------- */

/* ---------------------------------------------------------- */
/* Standard Windows console mode include files.               */
/* ---------------------------------------------------------- */

#include <windows.h>
#include <winioctl.h>
#include <time.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>


/* ---------------------------------------------------------- */
/* Application specific include files.                        */
/* ---------------------------------------------------------- */

#include "bluechip.h"


/* ---------------------------------------------------------- */
/* Program name and version number strings.                   */
/* ---------------------------------------------------------- */

#define PROGNAME  "PCI Data Acquisition PCI-DIO Test v"
#define VERSION   "1.00.00"
#define COPYRIGHT "(C)2000 Blue Chip Technology Ltd."
#define RIGHTS    "All rights reserved"


/* ---------------------------------------------------------- */
/* Handles and board ID structures.                           */
/* ---------------------------------------------------------- */

BCT_HANDLE nPciDioHandle;
BCT_BOARD_ID pPciDioBoardId;
PBCT_BUFFER pPciDioBuffer;


/* ---------------------------------------------------------- */
/* Define keystrokes.                                         */
/* ---------------------------------------------------------- */

#define	ESC 0x1B


/* ---------------------------------------------------------- */
/* Local function prototypes.                                 */
/* ---------------------------------------------------------- */

void Initialise(void);
void Deinitialise(void);
void DirectIO(void);
void PacerIO(void);


/* ---------------------------------------------------------- */
/* Main function.                                             */
/*                                                            */
/* Input  : None                                              */
/* Return : Error code if problems.                           */
/* ---------------------------------------------------------- */

int main(void)
{
	char cSelection;

	// Display the program header
	printf("%s%s\n%s\n%s\n", PROGNAME, VERSION, COPYRIGHT, RIGHTS);

	// Initialise the handles and the board ID structure.
	Initialise();

	// Display a menu and wait for a keypress
	do
	{
		printf("\n\n");
		printf("Please select a test:\n");
		printf("\t   1)  Perform direct IO loopback test\n");
		printf("\t   2)  Perform pacer controlled IO loopback test\n");
		printf("\t<ESC> Exit\n");
		cSelection = _getch();
		switch (cSelection)
		{
			case '1':
				DirectIO();
				break;
			case '2':
				PacerIO();
				break;
		}
	} while (cSelection != ESC);

	// We are done so close the handles and release the board ID
	Deinitialise();
}


/* ---------------------------------------------------------- */
/* Local function implementations.                            */
/* ---------------------------------------------------------- */

/* ---------------------------------------------------------- */
/* Initialise - Initialise the board id structure and the     */
/*              handles that we are going to use.             */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void Initialise(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];

	// Initialise a handle
	dwStatus = BCTInitHandle(&nPciDioHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitHandle returned: %s\n", szErrorText);
	}

	// Get the driver to fill out the board ID structure
	dwStatus = BCTGetBoardId(&pPciDioBoardId, PCI_DIO, 0);
	if (dwStatus != BCT_OK)
	{

		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTGetBoardId returned: %s\n", szErrorText);
	}

	// Initialise the PCI-DIO.  In the Windows NT 4 driver we used to call
	// BCTInitIsoDigModes to set up either 2 8 bit ports for the PCI-DIO
	// or 1 16bit port, or some two lots of 8 bit output and 1 16bit input
	// port, etc.  This was very involved and very complicated so we have
	// added a simpler function -> BCTInitPCIDio.  This allows one handle
	// to be used for 16bit bi-directional IO.  This function just requires
	// a board ID structure and a board index number.
	dwStatus = BCTInitPCIDio(&pPciDioBoardId, 0);     // We only have 1 board
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitPciDio returned: %s\n", szErrorText);
	}

	// Open the handle that will be used by the pacer
	dwStatus = BCTOpen(&nPciDioHandle, &pPciDioBoardId, BCT_ISODIG, 0, BCT_PORT_ISO);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTOpen returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* Deinitialise - Close the board id structure and the        */
/*                handles that we have been using.            */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void Deinitialise(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];

	// Make sure that the outputs are all zeroed before exit
	dwStatus = BCTWritePort16(&nPciDioHandle, 0x0000);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciDioHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTWritePort16 returned: %s\n", szErrorText);
	}

	// Close the handle
	dwStatus = BCTClose(&nPciDioHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTClose returned: %s\n", szErrorText);
	}

	// Release the board ID structure
	dwStatus = BCTReleaseBoardId(&pPciDioBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTReleaseBoardId returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* DirectIO - Perform the loopback test using direct IO calls */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void DirectIO(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	BCT_WORD wOutVal = 0;
	BCT_WORD wInVal = 0;
	char cKey;
	long int LIoutval;

	// Get a value from the user to output to the PIO
	do
	{
		printf("Please enter a 16bit value in hex to output to the PCI-DIO: ");
		scanf_s("%x", &LIoutval);
		wOutVal = (BCT_WORD) LIoutval;
	} while (wOutVal < 0x0000 && wOutVal > 0xFFFF);

	// Output the value to PCI-DIO
	dwStatus = BCTWritePort16(&nPciDioHandle, wOutVal);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciDioHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTWritePort16 returned: %s\n", szErrorText);
	}

	// In order to ensure that the value propogates through correctly
	// we need to ensure that we wait for 2.4ms before we read the
	// value back on the input.  This is to give the PCI-DIO time to 
	// serially propogate the output value to all of the output pins.
	// To make life simple we can just wait for a keystroke
	printf("Press a key to read back value\n");
	cKey = _getch();

	// Read it back to ensure that we got what we were expecting
	dwStatus = BCTReadPort16(&nPciDioHandle, &wInVal);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciDioHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTReadPort16 returned: %s\n", szErrorText);
	}

	// Comapre the values
	printf("Output 0x%.4x, read back 0x%.4x. %s\n", wOutVal, wInVal, wOutVal == wInVal ? "CORRECT" : "INCORRECT");
}


/* ---------------------------------------------------------- */
/* PacerIO - Perform the loopback test using the PCI-DIO      */
/*           under pacer control.                             */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void PacerIO(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	BCT_WORD wInVal;
	char cDummy;
	int nIndex;

	// Allocate some space for the output buffer and pre-load some values
	dwStatus = BCTAllocate(&pPciDioBuffer, 512);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAllocate returned: %s\n", szErrorText);
	}
	for (nIndex = 0; nIndex < 512; nIndex+=2)
	{
		pPciDioBuffer -> Buffer[nIndex] = (nIndex % 4) ? 0x55 : 0xAA;
		pPciDioBuffer -> Buffer[nIndex + 1] = (nIndex % 4) ? 0x55 : 0xAA;
	}

	// Tell the user what is going on.
	printf("Starting pacer...press any key to exit\n");

	// Initialise a pacer
	dwStatus = BCTInitPacer(&pPciDioBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitPacer returned: %s\n", szErrorText);
	}

	// Add an event to the pacers event list
	dwStatus = BCTAddPacerBlockIo(&nPciDioHandle, &pPciDioBoardId, BLOCK_REPEATED_WRITE, pPciDioBuffer, NULL, 512, 0);
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTAddPacerBlockIo returned: %s\n", szErrorText);
	}

	// Start the pacer itself
	dwStatus = BCTStartPacer(&pPciDioBoardId, 2500);    // Update at 2.5s interval
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTStartPacer returned: %s\n", szErrorText);
	}

	// Spin in a loop until the user presses a key
	while (!_kbhit())
	{
		dwStatus = BCTReadPort16(&nPciDioHandle, &wInVal);
		if (dwStatus == BCT_IO_PENDING)
		{
			dwStatus = BCTWait(&nPciDioHandle, INFINITE);
		}
		if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
		{
			BCTErr2Txt(dwStatus, szErrorText);
			printf("BCTReadPort returned: %s\n", szErrorText);
		}

		// Display the value we have read back
		printf("Value read from PCI-DIO: 0x%.4x\r", wInVal);
	}
	cDummy = _getch();             // Throw away the dummy keypress
	printf("\n");

	// Stop the pacer before we exit
	dwStatus = BCTStopPacer(&pPciDioBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTStopPacer returned: %s\n", szErrorText);
	}

	// Release the memory we used for the buffer
	dwStatus = BCTRelease(&pPciDioBuffer);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTRelease returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* End of listing.                                            */
/* ---------------------------------------------------------- */
