/* ---------------------------------------------------------- */
/* (C)2000 Blue Chip Technology Ltd.  All Rights Reserved.    */
/* ---------------------------------------------------------- */
/*                                                            */
/*  FileName  : $WorkFile::                                 $ */
/*                                                            */
/*  Release   : 1.00                                          */
/*                                                            */
/*  Contents  : This file contains the functions for the C    */
/*              based console mode application that will      */
/*              test the PCI-WDT                              */
/*                                                            */
/*  History   :                                               */
/*  $Revision::                                             $ */
/*  $Author::                                               $ */
/*  $Modtime::                                              $ */
/*  $History::                                              $ */
/*                                                            */
/* ---------------------------------------------------------- */

/* ---------------------------------------------------------- */
/* Standard Windows console mode include files.               */
/* ---------------------------------------------------------- */

#include <windows.h>
#include <winioctl.h>
#include <time.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>


/* ---------------------------------------------------------- */
/* Application specific include files.                        */
/* ---------------------------------------------------------- */

#include "bluechip.h"


/* ---------------------------------------------------------- */
/* Program name and version number strings.                   */
/* ---------------------------------------------------------- */

#define PROGNAME  "PCI Data Acquisition PCI-WDT Test Program v"
#define VERSION   "1.00.00"
#define COPYRIGHT "(C)2000 Blue Chip Technology Ltd."
#define RIGHTS    "All rights reserved"


/* ---------------------------------------------------------- */
/* Handles and board ID structures.                           */
/* ---------------------------------------------------------- */

BCT_HANDLE nPciWdtHandle;
BCT_BOARD_ID pPciWdtBoardId;
BCT_BYTE nTimerVal;


/* ---------------------------------------------------------- */
/* Define keystrokes.                                         */
/* ---------------------------------------------------------- */

#define	ESC 0x1B


/* ---------------------------------------------------------- */
/* Local function prototypes.                                 */
/* ---------------------------------------------------------- */

void Initialise(void);
void Deinitialise(void);
void ReadWdt(void);
void SetLimits(void);
void SetWdtTimer(void);
void ResetWdtTimer(void);
void SetEnableMask(void);
void SetOutputMask(void);
void ResetWdt(void);


/* ---------------------------------------------------------- */
/* Main function.                                             */
/*                                                            */
/* Input  : None                                              */
/* Return : Error code if problems.                           */
/* ---------------------------------------------------------- */

int main(void)
{
	char cSelection;

	// Display the program header
	printf("%s%s\n%s\n%s\n", PROGNAME, VERSION, COPYRIGHT, RIGHTS);

	// Initialise the handles and the board ID structure.
	Initialise();

	// Display a menu and wait for a keypress
	do
	{
		printf("\n\n");
		printf("Please select a test:\n");
		printf("\t   1)  Read the settings from the PCI-WDT\n");
		printf("\t   2)  Write voltage limits\n");
		printf("\t   3)  Enter watchdog timer value (hex)\n");
		printf("\t   4)  Reset timer\n");
		printf("\t   5)  Enter WDT Enable mask\n");
		printf("\t   6)  Enter WDT Output Mask\n");
		printf("\t   7)  Reset PCI-WDT\n");
		printf("\t<ESC> Exit\n");
		cSelection = _getch();
		switch (cSelection)
		{
			case '1':
				ReadWdt();
				break;
			case '2':
				SetLimits();
				break;
			case '3':
				SetWdtTimer();
				break;
			case '4':
				ResetWdtTimer();
				break;
			case '5':
				SetEnableMask();
				break;
			case '6':
				SetOutputMask();
				break;
			case '7':
				ResetWdt();
				break;
		}
	} while (cSelection != ESC);

	// We are done so close the handles and release the board ID
	Deinitialise();
}


/* ---------------------------------------------------------- */
/* Local function implementations.                            */
/* ---------------------------------------------------------- */

/* ---------------------------------------------------------- */
/* Initialise - Initialise the board id structure and the     */
/*              handles that we are going to use.             */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void Initialise(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];

	// Initialise the handle we are going to use
	dwStatus = BCTInitHandle(&nPciWdtHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTInitHandle returned: %s\n", szErrorText);
	}
	// Get the driver to fill out the board ID structure
	dwStatus = BCTGetBoardId(&pPciWdtBoardId, PCI_WDT, 0);
	if (dwStatus != BCT_OK)
	{

		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTGetBoardId returned: %s\n", szErrorText);
	}

	// Open the handle that will be used for accessing the watchdog
	dwStatus = BCTOpen(&nPciWdtHandle, &pPciWdtBoardId, BCT_WDT, 0, 0);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTOpen returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* Deinitialise - Close the board id structure and the        */
/*                handles that we have been using.            */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void Deinitialise(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];

	// Close the handle
	dwStatus = BCTClose(&nPciWdtHandle);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTClose returned: %s\n", szErrorText);
	}

	// Release the board ID structure
	dwStatus = BCTReleaseBoardId(&pPciWdtBoardId);
	if (dwStatus != BCT_OK)
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTReleaseBoardId returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* ReadWdt - Read the values for the voltage rails and the    */
/*           temperature back from the PCI-WDT                */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void ReadWdt(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	BCT_WDT_READDATA pReadData;

	// Read the inputs back from the watchdog
	dwStatus = BCTReadWdt(&nPciWdtHandle, &pReadData);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciWdtHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTReadWdt returned: %s\n", szErrorText);
	}

	// Display the readings
	printf("3.3V rail   : %6.2fV\n", pReadData.IN0_3_3V * 0.016);
	printf("  5V rail   : %6.2fV\n", pReadData.IN1_5V * 0.0268);
	printf(" 12V rail   : %6.2fV\n", pReadData.IN2_12V * 0.064);
	printf("-12V rail   : %6.2fV\n", pReadData.IN5_Minus12V * -0.064);
	printf("Temperature : %6dC\n", pReadData.Temperature);

}


/* ---------------------------------------------------------- */
/* SetLimits - Set the limits for the voltage rails on the    */
/*             PCI-WDT.                                       */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void SetLimits(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	BCT_WDT_SETDATA pSetData;

	// Tell the user what we are setting the limits to
	printf("Setting limits to:\n");
	printf("3.3V rail limits to  +/- 5%%\n");
	printf("  5V rail limits to  +/- 5%%\n");
	printf(" 12V rail limits to  +/- 5%%\n");
	printf("-12V rail limits to  +/- 5%%\n");

	// Enter the limits into the structure
	pSetData.IN0_High_3_3V = (unsigned char)((3.3 * 1.05) / 0.016);
	pSetData.IN0_Low_3_3V = (unsigned char)((3.3 * 0.95) / 0.016);
	pSetData.IN1_High_5V = (unsigned char)((5 * 1.05) / 0.0268);
	pSetData.IN1_Low_5V = (unsigned char)((5 * 0.95) / 0.0268);
	pSetData.IN2_High_12V = (unsigned char)((12 * 1.05) / 0.064);
	pSetData.IN2_Low_12V = (unsigned char)((12 * 0.95) / 0.064);
	pSetData.IN5_High_Minus12V = (unsigned char)((12 * 1.05) / 0.064);
	pSetData.IN5_Low_Minus12V = (unsigned char)((12 * 0.95) / 0.064);

	// Pass the limits to the LM78/9 on the PCI-WDT
	dwStatus = BCTSetWdt(&nPciWdtHandle, &pSetData);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciWdtHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTReadWdt returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* SetWdtTimer - Program the watchdog timer timeout period.   */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void SetWdtTimer(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	long int LOutputMask;

	// Get the timer value from the user
	do
	{
		printf("Enter the watchdog timer value (hex): ");
		scanf_s("%x", &LOutputMask);
		nTimerVal = (BCT_BYTE) LOutputMask;
	} while (nTimerVal < 0x00 && nTimerVal > 0xFF);
	printf("Setting timer value to 0x%x\n", nTimerVal);

	// Program the watchdog timer
	dwStatus = BCTWatchdog(&nPciWdtHandle, BCT_WD_WRITE_TIMEOUT, &nTimerVal);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciWdtHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTWatchdog returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* ResetWdtTimer - Refresh the watchdog timer so that we      */
/*                 don't timeout.                             */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void ResetWdtTimer(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];

	// Reset the timeout
	printf("Resetting timer...\n");
	dwStatus = BCTWatchdog(&nPciWdtHandle, BCT_WD_REFRESH_TIMEOUT, &nTimerVal);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciWdtHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTWatchdog returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* SetEnableMask - Set the mask for the devices that can      */
/*                 trigger a fault condition.                 */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void SetEnableMask(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	BCT_BYTE nEnabMask;
	long int LOutputMask;

	// Ask the customer for the value to set the enable mask
	do
	{
		printf("Enter the WDT Enable mask (hex): ");
		scanf_s("%x", &LOutputMask);
		nEnabMask = (BCT_BYTE) LOutputMask;
	} while (nEnabMask < 0x00 && nEnabMask > 0xFF);
	printf("Setting enable mask to 0x%x\n", nEnabMask);

	// Set the enable mask
	dwStatus = BCTWatchdog(&nPciWdtHandle, BCT_WD_WRITE_ENABLE_MASK, &nEnabMask);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciWdtHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTWatchdog returned: %s\n", szErrorText);
	}
}


// TODO: This needs the comments changing to make them all
//       less ambiguous.

/* ---------------------------------------------------------- */
/* SetOutputMask - Set the bit mask for the outputs to be     */
/*                 triggered in the event of an error.        */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void SetOutputMask(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	BCT_BYTE nOutputMask;
	long int LOutputMask;

	// Ask the customer for the value to set the output mask
	do
	{
		printf("Enter the WDT output mask (hex): ");
		scanf_s("%x", &LOutputMask);
		nOutputMask = (BCT_BYTE) LOutputMask;
	} while (nOutputMask < 0x00 && nOutputMask > 0xFF);
	printf("Setting enable mask to 0x%x\n", nOutputMask);

	// Set the output mask
	dwStatus = BCTWatchdog(&nPciWdtHandle, BCT_WD_WRITE_OUTPUT_MASK, &nOutputMask);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciWdtHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTWatchdog returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* ResetWdt - Reset the watchdog timer to disable all outputs */
/*            and inputs and to reset the onboard LM78.       */
/*                                                            */
/* Input  : None                                              */
/* Return : None                                              */
/* ---------------------------------------------------------- */

void ResetWdt(void)
{
	BCT_DWORD dwStatus;
	char szErrorText[120];
	BCT_BYTE nDummy;

	// Reset the PCI-WDT
	nDummy = 0x00;
	dwStatus = BCTWatchdog(&nPciWdtHandle, BCT_WD_RESET_BOARD, &nDummy);
	if (dwStatus == BCT_IO_PENDING)
	{
		dwStatus = BCTWait(&nPciWdtHandle, INFINITE);
	}
	if ((dwStatus != BCT_OK) && (dwStatus != BCT_IO_PENDING))
	{
		BCTErr2Txt(dwStatus, szErrorText);
		printf("BCTWatchdog returned: %s\n", szErrorText);
	}
}


/* ---------------------------------------------------------- */
/* End of listing.                                            */
/* ---------------------------------------------------------- */
